@@ -45,13 +45,21 @@ static const char *const s_vars   = "variables";
 
 #define TREE_HIDDEN(tree) ((tree)->label == NULL)
 
+#define AUGEAS_META_PATHX_FUNC AUGEAS_META_TREE "/version/pathx/functions"
+
 static const char *const static_nodes[][2] = {
     { AUGEAS_META_TREE "/version", PACKAGE_VERSION },
     { AUGEAS_META_TREE "/version/save/mode[1]", AUG_SAVE_BACKUP_TEXT },
     { AUGEAS_META_TREE "/version/save/mode[2]", AUG_SAVE_NEWFILE_TEXT },
     { AUGEAS_META_TREE "/version/save/mode[3]", AUG_SAVE_NOOP_TEXT },
     { AUGEAS_META_TREE "/version/save/mode[4]", AUG_SAVE_OVERWRITE_TEXT },
-    { AUGEAS_META_TREE "/version/defvar/expr", NULL }
+    { AUGEAS_META_TREE "/version/defvar/expr", NULL },
+    { AUGEAS_META_PATHX_FUNC "/count", NULL },
+    { AUGEAS_META_PATHX_FUNC "/glob", NULL },
+    { AUGEAS_META_PATHX_FUNC "/label", NULL },
+    { AUGEAS_META_PATHX_FUNC "/last", NULL },
+    { AUGEAS_META_PATHX_FUNC "/position", NULL },
+    { AUGEAS_META_PATHX_FUNC "/regexp", NULL }
 };
 
 static const char *const errcodes[] = {
@@ -64,7 +72,9 @@ static const char *const errcodes[] = {
     "Syntax error in lens definition",                  /* AUG_ESYNTAX */
     "Lens not found",                                   /* AUG_ENOLENS */
     "Multiple transforms",                              /* AUG_EMXFM */
-    "Node has no span info"                             /* AUG_ENOSPAN */
+    "Node has no span info",                            /* AUG_ENOSPAN */
+    "Cannot move node into its descendant",             /* AUG_EMVDESC */
+    "Failed to execute command"                         /* AUG_ECMDRUN */
 };
 
 static void tree_mark_dirty(struct tree *tree) {
@@ -189,21 +199,38 @@ int tree_set_value(struct tree *tree, const char *value) {
     return 0;
 }
 
+static void store_error(const struct augeas *aug, const char *label, const char *value,
+                 int nentries, ...) {
+    va_list ap;
+    struct tree *tree;
+
+    ensure(nentries % 2 == 0, aug);
+    tree = tree_path_cr(aug->origin, 3, s_augeas, s_error, label);
+    if (tree == NULL)
+        return;
+
+    tree_set_value(tree, value);
+
+    va_start(ap, nentries);
+    for (int i=0; i < nentries; i += 2) {
+        char *l = va_arg(ap, char *);
+        char *v = va_arg(ap, char *);
+        struct tree *t = tree_child_cr(tree, l);
+        if (t != NULL)
+            tree_set_value(t, v);
+    }
+    va_end(ap);
+ error:
+    return;
+}
+
 /* Report pathx errors in /augeas/pathx/error */
 static void store_pathx_error(const struct augeas *aug) {
     if (aug->error->code != AUG_EPATHX)
         return;
 
-    struct tree *error =
-        tree_path_cr(aug->origin, 3, s_augeas, s_pathx, s_error);
-    if (error == NULL)
-        return;
-    tree_set_value(error, aug->error->minor_details);
-
-    struct tree *tpos = tree_child_cr(error, s_pos);
-    if (tpos == NULL)
-        return;
-    tree_set_value(tpos, aug->error->details);
+    store_error(aug, s_pathx, aug->error->minor_details,
+                2, s_pos, aug->error->details);
 }
 
 struct pathx *pathx_aug_parse(const struct augeas *aug,
@@ -318,7 +345,7 @@ static void restore_locale(ATTRIBUTE_UNUSED struct augeas *aug) { }
  * that count is 0. That requires that all public functions enclose their
  * work within a matching pair of api_entry/api_exit calls.
  */
-static void api_entry(const struct augeas *aug) {
+void api_entry(const struct augeas *aug) {
     struct error *err = ((struct augeas *) aug)->error;
 
     ((struct augeas *) aug)->api_entries += 1;
@@ -330,7 +357,7 @@ static void api_entry(const struct augeas *aug) {
     save_locale((struct augeas *) aug);
 }
 
-static void api_exit(const struct augeas *aug) {
+void api_exit(const struct augeas *aug) {
     assert(aug->api_entries > 0);
     ((struct augeas *) aug)->api_entries -= 1;
     if (aug->api_entries == 0) {
@@ -352,6 +379,12 @@ struct augeas *aug_init(const char *root, const char *loadpath,
         goto error;
     if (ALLOC(result->error) < 0)
         goto error;
+    if (make_ref(result->error->info) < 0)
+        goto error;
+    result->error->info->error = result->error;
+    result->error->info->filename = dup_string("(unknown file)");
+    if (result->error->info->filename == NULL)
+        goto error;
     result->error->aug = result;
 
     result->origin = make_tree_origin(tree_root);
@@ -1083,10 +1116,10 @@ int aug_mv(struct augeas *aug, const char *src, const char *dst) {
     /* Don't move SRC into its own descendent */
     t = td;
     do {
-        if (t == ts)
-            goto error;
+        ERR_THROW(t == ts, aug, AUG_EMVDESC,
+                  "destination %s is a descendant of %s", dst, src);
         t = t->parent;
-    } while (! ROOT_P(t));
+    } while (t != aug->origin);
 
     free_tree(td->children);
 
@@ -1446,6 +1479,7 @@ void aug_close(struct augeas *aug) {
     free((void *) aug->root);
     free(aug->modpathz);
     free_symtab(aug->symtab);
+    unref(aug->error->info, info);
     free(aug->error->details);
     free(aug->error);
     free(aug);
@@ -1482,7 +1516,7 @@ int aug_error(struct augeas *aug) {
 const char *aug_error_message(struct augeas *aug) {
     aug_errcode_t errcode = aug->error->code;
 
-    if (errcode > ARRAY_CARDINALITY(errcodes))
+    if (errcode >= ARRAY_CARDINALITY(errcodes))
         errcode = AUG_EINTERNAL;
     return errcodes[errcode];
 }
