@@ -78,6 +78,19 @@ static char *token_to_eol(char *s, char **tok) {
     return s;
 }
 
+static char *findpath(char *s, char **p) {
+    char *t = skipws(s);
+
+    while (*s && *s != '=') s++;
+    if (s > t) {
+        s -= 1;
+        while (*s && isspace(*s)) s -= 1;
+        s += 1;
+    }
+    *p = strndup(t, s - t);
+    return s;
+}
+
 static struct test *read_tests(void) {
     char *fname;
     FILE *fp;
@@ -107,7 +120,7 @@ static struct test *read_tests(void) {
             if (ALLOC(e) < 0)
                 die("out of memory");
             list_append(t->entries, e);
-            s = token(s, &(e->path));
+            s = findpath(s, &(e->path));
             s = skipws(s);
             if (*s) {
                 if (*s != '=') {
@@ -301,7 +314,7 @@ static int test_invalid_regexp(struct augeas *aug) {
     return -1;
 }
 
-static int run_tests(struct test *tests) {
+static int run_tests(struct test *tests, int argc, char **argv) {
     char *lensdir;
     struct augeas *aug = NULL;
     int r, result = EXIT_SUCCESS;
@@ -318,30 +331,40 @@ static int run_tests(struct test *tests) {
     r = aug_defvar(aug, "localhost", "'127.0.0.1'");
     if (r != 0)
         die("aug_defvar $localhost");
+    r = aug_defvar(aug, "php", "/files/etc/php.ini");
+    if (r != 1)
+        die("aug_defvar $php");
 
     list_for_each(t, tests) {
+        int skip = (argc > 0);
+        for (int i=0; i < argc; i++)
+            if (STREQ(argv[i], t->name))
+                skip = 0;
+        if (skip)
+            continue;
         if (run_one_test(aug, t) < 0)
             result = EXIT_FAILURE;
     }
 
-    if (test_rm_var(aug) < 0)
-        result = EXIT_FAILURE;
+    if (argc == 0) {
+        if (test_rm_var(aug) < 0)
+            result = EXIT_FAILURE;
 
-    if (test_defvar_nonexistent(aug) < 0)
-        result = EXIT_FAILURE;
+        if (test_defvar_nonexistent(aug) < 0)
+            result = EXIT_FAILURE;
 
-    if (test_defnode_nonexistent(aug) < 0)
-        result = EXIT_FAILURE;
-
-    if (test_invalid_regexp(aug) < 0)
-        result = EXIT_FAILURE;
+        if (test_defnode_nonexistent(aug) < 0)
+            result = EXIT_FAILURE;
 
+        if (test_invalid_regexp(aug) < 0)
+            result = EXIT_FAILURE;
+    }
     aug_close(aug);
 
     return result;
 }
 
-int main(void) {
+int main(int argc, char **argv) {
     struct test *tests;
 
     abs_top_srcdir = getenv("abs_top_srcdir");
@@ -353,7 +376,7 @@ int main(void) {
     }
 
     tests = read_tests();
-    return run_tests(tests);
+    return run_tests(tests, argc - 1, argv + 1);
     /*
     list_for_each(t, tests) {
         printf("Test %s\n", t->name);
